//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
//Praktická Elektronika - MSP430 seriál, časť VII.
//Druhý príklad: pripojenie LCD a I2C (PCF8574)
//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
#include <LiquidCrystal.h>                             //(1) LCD communication library
#include <Wire.h>                                      //(2) I2C communication library

LiquidCrystal lcd(P1_0, P1_1, P1_2, P1_3, P1_4, P1_5); //(3) define the connection to LCD
short v_counter = 0;                                   //(4) define counter variable
short i2c_trans_stat;                                  //(5) I2C communication result
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void setup()                                           //(6) Setup function
{
  lcd.begin(20, 4);                                    //(7) set-up usage of 4 rows x 20 columns
  lcd.setCursor(0,0);                                  //(8) put cursor to position col:0 / row:0
  lcd.print("MSP430 Basics");                          //(9) write the text in quotes starting at 0,0
  lcd.setCursor(0,1);                                  //(10) put cursor to position col:0 / row:1
  lcd.print("MCU: MSP430G2553");                       //(11) write the text in quotes starting at 0,1
  lcd.setCursor(0,2);                                  //(12) put cursor to position col:0 / row:2
  lcd.print("Example 8: LCD+I2C");                     //(13) write the text in quotes starting at 0,2
  lcd.setCursor(0,3);                                  //(14) put cursor to position col:0 / row:3
  lcd.print("v1.03 / 27.03.2021");                     //(15) write the text in quotes starting at 0,3

  delay(3000);                                         //(16) wait 3 sec
  lcd.clear();                                         //(17) clear the content of the LCD
  Wire.begin();                                        //(18) I2C initialization
  v_counter = 255;                                     //(19) counter initialization
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void loop()                                            //(20) main loop starts here
{
  write_texts();                                       //(21) write static texts
  pcf8574_send(v_counter);                             //(22) send value of the v_counter to PCF8574
  w_bin8(v_counter);                                   //(23) show counter information
  w_ack_info();                                        //(24) show result of I2C communication
  
  v_counter--;                                         //(25) decrement counter
  delay(200);                                          //(26) wait 0,2 sec
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void write_texts()                                     //(27) start of write_texts function
{
  lcd.setCursor(0,0);                                  //(28) put cursor to position col:0 / row:0
  lcd.print("First I2C project");                      //(29) write the text in quotes starting at 0,0
  lcd.setCursor(0,1);                                  //(30) put cursor to position col:0 / row:1
  lcd.print("Energia: PCF8574+LCD");                   //(31) write the text in quotes starting at 0,1
  lcd.setCursor(0,2);                                  //(32) put cursor to position col:0 / row:2
  lcd.print("Counter:");                               //(33) write the text in quotes starting at 0,2
  lcd.setCursor(0,3);                                  //(34) put cursor to position col:0 / row:3
  lcd.print("Ad:");                                    //(35) write the text in quotes starting at 0,3
  lcd.setCursor(11,3);                                 //(36) put cursor to position col:11 / row:3
  lcd.print("D:");                                     //(37) write the text in quotes starting at 11,3
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void w_bin8(short i_byte)                              //(38) writing counter stage - binary
{
  lcd.setCursor(9,2);                                  //(39) put cursor to position col:9 / row:2
  if ((i_byte & 0x80) == 0) lcd.print("0");            //(40) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(41) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x40) == 0) lcd.print("0");            //(42) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(43) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x20) == 0) lcd.print("0");            //(44) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(45) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x10) == 0) lcd.print("0");            //(46) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(47) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x08) == 0) lcd.print("0");            //(48) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(49) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x04) == 0) lcd.print("0");            //(50) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(51) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x02) == 0) lcd.print("0");            //(52) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(53) if the corresponding bit is not 0 -> write 1
  if ((i_byte & 0x01) == 0) lcd.print("0");            //(54) if the corresponding bit is 0 -> write 0
  else                      lcd.print("1");            //(55) if the corresponding bit is not 0 -> write 1
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void w_ack_info()                                      //(56) show info about the communication status
{
  switch(i2c_trans_stat)                               //(57) based on variable i2c_trans_stat we expect 3 options
  {
    case 0:                                            //(58) everything went ok
       lcd.setCursor(3,3);                             //(59) set cursor to desired posiion on LCD
       lcd.print("ACK    ");                           //(60) write text
       lcd.setCursor(13,3);                            //(61) set cursor to desired posiion on LCD
       lcd.print("ACK    ");                           //(62) write text
       break;                                          //(63) done
    case 2:
       lcd.setCursor(3,3);                             //(64) set cursor to desired posiion on LCD
       lcd.print("not ack");                           //(65) write text
       lcd.setCursor(13,3);                            //(66) set cursor to desired posiion on LCD
       lcd.print("not ack");                           //(67) write text
       break;                                          //(68) done
    case 3:
       lcd.setCursor(3,3);                             //(69) set cursor to desired posiion on LCD
       lcd.print("ACK    ");                           //(70) write text
       lcd.setCursor(13,3);                            //(71) set cursor to desired posiion on LCD
       lcd.print("not ack");                           //(72) write text
       break;                                          //(73) done
  }
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
void pcf8574_send(short i_byte)                       //(74) send data to PCF8574 via I2C
{
  Wire.beginTransmission(0b00100000);                 //(75) start communication expecting the chip on address 40h
  Wire.write(i_byte);                                 //(76) send the value of the counter
  i2c_trans_stat = Wire.endTransmission();            //(77) close communication and get the status
}
// ----------------------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------------------
